use core::cell::Cell;
use crate::cell::CellView;
use crate::transducer::Stateful;

pub trait Poll<T: Copy> {
    async fn poll() -> T;
}

pub struct RawStatefulInput<T: Copy> {
    pub state_cell: Cell<T>,
}

impl<T: Copy> RawStatefulInput<T> {
    #[inline(always)]
    pub fn new(state_cell: Cell<T>) -> Self {
        Self { state_cell }
    }

    #[inline(always)]
    pub fn raw_update(&self, value: T) {
        self.state_cell.set(value);
    }
}

impl<T: Copy> Stateful for RawStatefulInput<T> {
    type Value = T;

    #[inline(always)]
    fn state_cell(&self) -> CellView<Self::Value> {
        (&self.state_cell).into()
    }

    #[inline(always)]
    fn state(&self) -> Self::Value {
        self.state_cell.get()
    }
}
